"""
API for libpaste.  Add new classes then add them to the list at the bottom.
"""
import posixpath
import urllib2
import re

class BaseConnector(object):
    def __init__(self, url):
        self.url = url

    @classmethod
    def can_handle(cls, url):
        """Override this to return True if the subclass can understand this url"""
        raise NotImplementedError()

    def raw_data(self):
        """Override if raw_url is not a valid text/plain equivalent"""
        fo = urllib2.urlopen(self.raw_url())
        data = fo.read()
        fo.close()
        return data

class RafbConnector(BaseConnector):
    """Handler for rafb.net urls like
    http://rafb.net/p/9gtiql52.html
    http://rafb.net/p/9gtiql52.txt
    Sends a proper 404 on missing pastes.
    """
    @classmethod
    def can_handle(cls, url):
        return "rafb.net" in url

    def raw_url(self):
        if self.url.endswith(".html"):
            # Turn to a plain text one
            return self.url[:-len(".html")] + ".txt"
        return self.url

class DpasteConnector(BaseConnector):
    """Handler for dpaste.com urls like
    http://dpaste.com/38647/
    http://dpaste.com/38647/plain/
    """
    @classmethod
    def can_handle(cls, url):
        return "dpaste.com" in url

    def raw_url(self):
        if "plain" not in self.url:
            return posixpath.join(url, "plain/")
        return self.url

class PastebizConnector(BaseConnector):
    """Handler for paste.biz urls like
    http://paste.biz/paste-3011.html
    http://paste.biz/dl/3011
    If missing, raw gives "Pastebin entry %s is not available"
    """
    @classmethod
    def can_handle(cls, url):
        return "paste.biz" in url

    def raw_url(self):
        m = re.search(r"paste-(\d+)", self.url)
        if m:
            return "http://paste.biz/dl/%s" % m.group(1)
        return self.url

class PastebinCaConnector(BaseConnector):
    """Handler for pastebin.ca urls like
    http://pastebin.ca/935437
    http://pastebin.ca/raw/935437
    If missing, raw gives "Invalid post ID."  Probably also gives a 404.
    """
    @classmethod
    def can_handle(cls, url):
        return "pastebin.ca" in url

    def raw_url(self):
        m = re.search(r"/\d+", self.url)
        if m:
            return "http://pastebin.ca/raw%s" % m.group(0)
        return self.url

class PastebinComConnector(BaseConnector):
    """Handler for pastebin.com urls like
    http://pastebin.com/m5d2995af
    http://pastebin.com/pastebin.php?dl=m5d2995af
    If missing, raw gives "Pastebin entry %s is not available." in a 404.
    """
    @classmethod
    def can_handle(cls, url):
        return "pastebin.com" in url

    def raw_url(self):
        m = re.search("/([^/]+)$", self.url)
        if m:
            return "http://pastebin.com/pastebin.php?dl=%s" % m.group(1)
        return self.url

class LisppasteConnector(BaseConnector):
    """Handler for paste.lisp.org urls like
    http://paste.lisp.org/display/55588
    http://paste.lisp.org/display/55588/raw
    If missing, raw gives "nil"
    """
    @classmethod
    def can_handle(cls, url):
        return "paste.lisp.org" in url

    def raw_url(self):
        m = re.search(r"display/(\d+)", self.url)
        if m:
            return "http://paste.lisp.org/display/%s/raw" % m.group(1)
        return self.url

class PocooConnector(BaseConnector):
    """Handler for paste.pocoo.org urls like
    http://paste.pocoo.org/show/31485/
    http://paste.pocoo.org/raw/31485/
    """
    @classmethod
    def can_handle(cls, url):
        return "paste.pocoo.org" in url

    def raw_url(self):
        m = re.search(r"show/(\d+)", self.url)
        if m:
            return "http://paste.pocoo.org/raw/%s/" % m.group(1)
        return self.url


pastebins = [
    RafbConnector, DpasteConnector, PastebizConnector, PastebinCaConnector,
    PastebinComConnector, LisppasteConnector, PocooConnector
]
